# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.search_items import SearchItems
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class SearchController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(SearchController, self).__init__(config)

    def search(self,
               q,
               mtype,
               market=None,
               limit=20,
               offset=0,
               include_external=None):
        """Does a GET request to /search.

        Get Spotify catalog information about albums, artists, playlists,
        tracks, shows, episodes or audiobooks
        that match a keyword string. Audiobooks are only available within the
        US, UK, Canada, Ireland, New Zealand and Australia markets.

        Args:
            q (str): The request query parameter.
            mtype (List[ItemTypeEnum]): The request query parameter.
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0
            include_external (IncludeExternalEnum, optional): The request
                query parameter.

        Returns:
            SearchItems: Response from the API. Search response

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/search')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('q')
                         .value(q))
            .query_param(Parameter()
                         .key('type')
                         .value(mtype))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .query_param(Parameter()
                         .key('include_external')
                         .value(include_external))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SearchItems.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

import warnings
from enum import Enum
from apimatic_core.http.configurations.http_client_configuration import HttpClientConfiguration
from apimatic_requests_client_adapter.requests_client import RequestsClient


class Environment(Enum):
    """An enum for SDK environments"""
    PRODUCTION = 0


class Server(Enum):
    """An enum for API servers"""
    DEFAULT = 0
    AUTH_SERVER = 1


class Configuration(HttpClientConfiguration):
    """A class used for configuring the SDK by a user.
    """

    @property
    def environment(self):
        return self._environment

    @property
    def o_auth_client_id(self):
        return self._authorization_code_auth_credentials.o_auth_client_id

    @property
    def o_auth_client_secret(self):
        return self._authorization_code_auth_credentials.o_auth_client_secret

    @property
    def o_auth_redirect_uri(self):
        return self._authorization_code_auth_credentials.o_auth_redirect_uri

    @property
    def o_auth_scopes(self):
        return self._authorization_code_auth_credentials.o_auth_scopes

    @property
    def o_auth_token(self):
        return self._authorization_code_auth_credentials.o_auth_token

    @property
    def authorization_code_auth_credentials(self):
        return self._authorization_code_auth_credentials

    def __init__(self, http_client_instance=None,
                 override_http_client_configuration=False, http_call_back=None,
                 timeout=60, max_retries=0, backoff_factor=2,
                 retry_statuses=None, retry_methods=None,
                 environment=Environment.PRODUCTION, o_auth_client_id=None,
                 o_auth_client_secret=None, o_auth_redirect_uri=None,
                 o_auth_token=None, o_auth_scopes=None,
                 authorization_code_auth_credentials=None):
        if retry_methods is None:
            retry_methods = ['GET', 'PUT']

        if retry_statuses is None:
            retry_statuses = [408, 413, 429, 500, 502, 503, 504, 521, 522, 524]

        super().__init__(http_client_instance,
                         override_http_client_configuration, http_call_back,
                         timeout, max_retries, backoff_factor, retry_statuses,
                         retry_methods)

        # Current API environment
        self._environment = environment

        self._authorization_code_auth_credentials = self.create_auth_credentials_object(
            o_auth_client_id, o_auth_client_secret, o_auth_redirect_uri,
            o_auth_token, o_auth_scopes, authorization_code_auth_credentials)

        # The Http Client to use for making requests.
        self.set_http_client(self.create_http_client())

    def clone_with(self, http_client_instance=None,
                   override_http_client_configuration=None, http_call_back=None,
                   timeout=None, max_retries=None, backoff_factor=None,
                   retry_statuses=None, retry_methods=None, environment=None,
                   o_auth_client_id=None, o_auth_client_secret=None,
                   o_auth_redirect_uri=None, o_auth_token=None,
                   o_auth_scopes=None, authorization_code_auth_credentials=None):
        http_client_instance = http_client_instance or self.http_client_instance
        override_http_client_configuration = override_http_client_configuration or self.override_http_client_configuration
        http_call_back = http_call_back or self.http_callback
        timeout = timeout or self.timeout
        max_retries = max_retries or self.max_retries
        backoff_factor = backoff_factor or self.backoff_factor
        retry_statuses = retry_statuses or self.retry_statuses
        retry_methods = retry_methods or self.retry_methods
        environment = environment or self.environment
        authorization_code_auth_credentials = self.create_auth_credentials_object(
            o_auth_client_id, o_auth_client_secret, o_auth_redirect_uri,
            o_auth_token, o_auth_scopes,
            authorization_code_auth_credentials or self.authorization_code_auth_credentials,
            stack_level=3)
        return Configuration(
            http_client_instance=http_client_instance,
            override_http_client_configuration=override_http_client_configuration,
            http_call_back=http_call_back, timeout=timeout, max_retries=max_retries,
            backoff_factor=backoff_factor, retry_statuses=retry_statuses,
            retry_methods=retry_methods, environment=environment,
            authorization_code_auth_credentials=authorization_code_auth_credentials
        )

    def create_http_client(self):
        return RequestsClient(
            timeout=self.timeout, max_retries=self.max_retries,
            backoff_factor=self.backoff_factor, retry_statuses=self.retry_statuses,
            retry_methods=self.retry_methods,
            http_client_instance=self.http_client_instance,
            override_http_client_configuration=self.override_http_client_configuration,
            response_factory=self.http_response_factory
        )

    # All the environments the SDK can run in
    environments = {
        Environment.PRODUCTION: {
            Server.DEFAULT: 'https://api.spotify.com/v1',
            Server.AUTH_SERVER: 'https://accounts.spotify.com'
        }
    }

    def get_base_uri(self, server=Server.DEFAULT):
        """Generates the appropriate base URI for the environment and the
        server.

        Args:
            server (Configuration.Server): The server enum for which the base
            URI is required.

        Returns:
            String: The base URI.

        """
        return self.environments[self.environment][server]

    @staticmethod
    def create_auth_credentials_object(o_auth_client_id, o_auth_client_secret,
                                       o_auth_redirect_uri, o_auth_token,
                                       o_auth_scopes,
                                       authorization_code_auth_credentials,
                                       stack_level=4):
        if o_auth_client_id is None \
                and o_auth_client_secret is None \
                and o_auth_redirect_uri is None \
                and o_auth_token is None \
                and o_auth_scopes is None:
            return authorization_code_auth_credentials

        warnings.warn(message=('The \'o_auth_client_id\', \'o_auth_client_secre'
                               't\', \'o_auth_redirect_uri\', \'o_auth_token\','
                               ' \'o_auth_scopes\' params are deprecated. Use '
                               '\'authorization_code_auth_credentials\' param i'
                               'nstead.'),
                      category=DeprecationWarning,
                      stacklevel=stack_level)

        if authorization_code_auth_credentials is not None:
            return authorization_code_auth_credentials.clone_with(
                o_auth_client_id, o_auth_client_secret, o_auth_redirect_uri,
                o_auth_token, o_auth_scopes)

        from spotifywebapiwithfixesandimprovementsfromsonallux.http.auth.o_auth_2 import AuthorizationCodeAuthCredentials
        return AuthorizationCodeAuthCredentials(o_auth_client_id,
                                                o_auth_client_secret,
                                                o_auth_redirect_uri,
                                                o_auth_token, o_auth_scopes)



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class RecommendationSeedObject(object):

    """Implementation of the 'RecommendationSeedObject' model.

    Attributes:
        after_filtering_size (int): The number of tracks available after
            min\_\* and max\_\* filters have been applied.
        after_relinking_size (int): The number of tracks available after
            relinking for regional availability.
        href (str): A link to the full track or artist data for this seed. For
            tracks this will be a link to a Track Object. For artists a link
            to an Artist Object. For genre seeds, this value will be `null`.
        id (str): The id used to select this seed. This will be the same as
            the string used in the `seed_artists`, `seed_tracks` or
            `seed_genres` parameter.
        initial_pool_size (int): The number of recommended tracks available
            for this seed.
        mtype (str): The entity type of this seed. One of `artist`, `track` or
            `genre`.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "after_filtering_size": 'afterFilteringSize',
        "after_relinking_size": 'afterRelinkingSize',
        "href": 'href',
        "id": 'id',
        "initial_pool_size": 'initialPoolSize',
        "mtype": 'type'
    }

    _optionals = [
        'after_filtering_size',
        'after_relinking_size',
        'href',
        'id',
        'initial_pool_size',
        'mtype',
    ]

    def __init__(self,
                 after_filtering_size=APIHelper.SKIP,
                 after_relinking_size=APIHelper.SKIP,
                 href=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 initial_pool_size=APIHelper.SKIP,
                 mtype=APIHelper.SKIP):
        """Constructor for the RecommendationSeedObject class"""

        # Initialize members of the class
        if after_filtering_size is not APIHelper.SKIP:
            self.after_filtering_size = after_filtering_size 
        if after_relinking_size is not APIHelper.SKIP:
            self.after_relinking_size = after_relinking_size 
        if href is not APIHelper.SKIP:
            self.href = href 
        if id is not APIHelper.SKIP:
            self.id = id 
        if initial_pool_size is not APIHelper.SKIP:
            self.initial_pool_size = initial_pool_size 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        after_filtering_size = dictionary.get("afterFilteringSize") if dictionary.get("afterFilteringSize") else APIHelper.SKIP
        after_relinking_size = dictionary.get("afterRelinkingSize") if dictionary.get("afterRelinkingSize") else APIHelper.SKIP
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        initial_pool_size = dictionary.get("initialPoolSize") if dictionary.get("initialPoolSize") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        # Return an object of this model
        return cls(after_filtering_size,
                   after_relinking_size,
                   href,
                   id,
                   initial_pool_size,
                   mtype)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'after_filtering_size={(self.after_filtering_size if hasattr(self, "after_filtering_size") else None)!r}, '
                f'after_relinking_size={(self.after_relinking_size if hasattr(self, "after_relinking_size") else None)!r}, '
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'initial_pool_size={(self.initial_pool_size if hasattr(self, "initial_pool_size") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'after_filtering_size={(self.after_filtering_size if hasattr(self, "after_filtering_size") else None)!s}, '
                f'after_relinking_size={(self.after_relinking_size if hasattr(self, "after_relinking_size") else None)!s}, '
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'initial_pool_size={(self.initial_pool_size if hasattr(self, "initial_pool_size") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.album_object import AlbumObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_albums import ManyAlbums
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_track_object import PagingSimplifiedTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_saved_album_object import PagingSavedAlbumObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paged_albums import PagedAlbums
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class AlbumsController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(AlbumsController, self).__init__(config)

    def get_an_album(self,
                     id,
                     market=None):
        """Does a GET request to /albums/{id}.

        Get Spotify catalog information for a single album.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.

        Returns:
            AlbumObject: Response from the API. An album

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/albums/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(AlbumObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_multiple_albums(self,
                            ids,
                            market=None):
        """Does a GET request to /albums.

        Get Spotify catalog information for multiple albums identified by
        their Spotify IDs.

        Args:
            ids (str): The request query parameter.
            market (str, optional): The request query parameter.

        Returns:
            ManyAlbums: Response from the API. A set of albums

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/albums')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyAlbums.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_an_albums_tracks(self,
                             id,
                             market=None,
                             limit=20,
                             offset=0):
        """Does a GET request to /albums/{id}/tracks.

        Get Spotify catalog information about an album’s tracks.
        Optional parameters can be used to limit the number of tracks returned.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingSimplifiedTrackObject: Response from the API. Pages of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/albums/{id}/tracks')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSimplifiedTrackObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_saved_albums(self,
                               limit=20,
                               offset=0,
                               market=None):
        """Does a GET request to /me/albums.

        Get a list of the albums saved in the current Spotify user's 'Your
        Music' library.

        Args:
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0
            market (str, optional): The request query parameter.

        Returns:
            PagingSavedAlbumObject: Response from the API. Pages of albums

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/albums')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSavedAlbumObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def save_albums_user(self,
                         ids,
                         body=None):
        """Does a PUT request to /me/albums.

        Save one or more albums to the current user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.
            body (MeAlbumsRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. The album is saved

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/albums')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def remove_albums_user(self,
                           ids,
                           body=None):
        """Does a DELETE request to /me/albums.

        Remove one or more albums from the current user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.
            body (MeAlbumsRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Album(s) have been removed from the
                library

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/albums')
            .http_method(HttpMethodEnum.DELETE)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def check_users_saved_albums(self,
                                 ids):
        """Does a GET request to /me/albums/contains.

        Check if one or more albums is already saved in the current Spotify
        user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/albums/contains')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_new_releases(self,
                         limit=20,
                         offset=0):
        """Does a GET request to /browse/new-releases.

        Get a list of new album releases featured in Spotify (shown, for
        example, on a Spotify player’s “Browse” tab).

        Args:
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagedAlbums: Response from the API. A paged set of albums

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/browse/new-releases')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagedAlbums.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.audio_features_object import AudioFeaturesObject


class ManyAudioFeatures(object):

    """Implementation of the 'ManyAudioFeatures' model.

    Attributes:
        audio_features (List[AudioFeaturesObject]): The model property of type
            List[AudioFeaturesObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "audio_features": 'audio_features'
    }

    def __init__(self,
                 audio_features=None):
        """Constructor for the ManyAudioFeatures class"""

        # Initialize members of the class
        self.audio_features = audio_features 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        audio_features = None
        if dictionary.get('audio_features') is not None:
            audio_features = [AudioFeaturesObject.from_dictionary(x) for x in dictionary.get('audio_features')]
        # Return an object of this model
        return cls(audio_features)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'audio_features={self.audio_features!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'audio_features={self.audio_features!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.authentication.header_auth import HeaderAuth
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.models.o_auth_token import OAuthToken
from apimatic_core.utilities.auth_helper import AuthHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.o_auth_authorization_controller import\
    OAuthAuthorizationController


class OAuth2(HeaderAuth):

    @property
    def error_message(self):
        """Display error message on occurrence of authentication failure
        in AuthorizationCodeAuth

        """
        return "AuthorizationCodeAuth: OAuthToken is undefined or expired."

    def __init__(self, authorization_code_auth_credentials, config):
        self._o_auth_client_id = authorization_code_auth_credentials.o_auth_client_id \
            if authorization_code_auth_credentials is not None else None
        self._o_auth_client_secret = authorization_code_auth_credentials.o_auth_client_secret \
            if authorization_code_auth_credentials is not None else None
        self._o_auth_redirect_uri = authorization_code_auth_credentials.o_auth_redirect_uri \
            if authorization_code_auth_credentials is not None else None
        if authorization_code_auth_credentials is not None \
                and isinstance(authorization_code_auth_credentials.o_auth_token, OAuthToken):
            self._o_auth_token = OAuthToken.from_dictionary(
                APIHelper.to_dictionary(authorization_code_auth_credentials.o_auth_token))
        else:
            self._o_auth_token = authorization_code_auth_credentials.o_auth_token \
                if authorization_code_auth_credentials is not None else None
        if authorization_code_auth_credentials is not None \
                and isinstance(authorization_code_auth_credentials.o_auth_scopes, list):
            self._o_auth_scopes = authorization_code_auth_credentials.o_auth_scopes
        else:
            self._o_auth_scopes = None
        self._config = config
        self._o_auth_api = OAuthAuthorizationController(config)
        auth_params = {}
        if isinstance(self._o_auth_token, OAuthToken) and hasattr(self._o_auth_token, 'access_token'):
            auth_params = {"Authorization": "Bearer {}".format(self._o_auth_token.access_token)}
        super().__init__(auth_params=auth_params)

    def is_valid(self):
        return (self._o_auth_token and isinstance(self._o_auth_token, OAuthToken)
                and not self.is_token_expired(self._o_auth_token))

    def get_authorization_url(self, state=None, additional_params=None):
        """ Builds and returns an authorization URL. The user is expected to
            obtain an authorization code from this URL and then call the authorize
            function with that authorization code.

        Args:
            state (str): An opaque state string.
            additional_params (dict): Any additional query parameters to be added to the URL.

        Returns:
            str: The authorization URL.

        """
        auth_url = self._config.get_base_uri(Server.AUTH_SERVER)
        auth_url += '/authorize'
        query_params = {
            'response_type': 'code',
            'client_id': self._o_auth_client_id,
            'redirect_uri': self._o_auth_redirect_uri
        }
        if self._o_auth_scopes:
            query_params['scope'] = ' '.join(self._o_auth_scopes)
        if state:
            query_params['state'] = state
        if additional_params:
            query_params.update(additional_params)
        auth_url = APIHelper.append_url_with_query_parameters(auth_url, query_params)
        return APIHelper.clean_url(auth_url)

    def build_basic_auth_header(self):
        """ Builds the basic auth header for endpoints in the
            OAuth Authorization Controller.

        Returns:
            str: The value of the Authentication header.

        """
        return "Basic {}".format(AuthHelper.get_base64_encoded_value(
            self._o_auth_client_id, self._o_auth_client_secret))

    def fetch_token(self, auth_code, additional_params=None):
        """ Authorizes the client.

            auth_code (str): The authentication code.
            additional_params (dict):  Any additional form parameters.

        Returns:
            OAuthToken: The OAuth token.

        """
        token = self._o_auth_api.request_token(
            self.build_basic_auth_header(),
            auth_code,
            self._o_auth_redirect_uri,
            _optional_form_parameters=additional_params
        )
        if hasattr(token, 'expires_in'):
            current_utc_timestamp = AuthHelper.get_current_utc_timestamp()
            token.expiry = AuthHelper.get_token_expiry(current_utc_timestamp, token.expires_in)
        return token

    def is_token_expired(self, o_auth_token=None):
        """ Checks if OAuth token has expired.

        Args:
            o_auth_token (OAuthToken): The OAuth token whose expiry is to be checked.

        Returns:
            bool: True if OAuth token has expired, False otherwise.

        """
        if o_auth_token is None:
            return (hasattr(self._o_auth_token, 'expiry')
                    and AuthHelper.is_token_expired(self._o_auth_token.expiry))

        return (hasattr(o_auth_token, 'expiry')
                and AuthHelper.is_token_expired(o_auth_token.expiry))

    def refresh_token(self, additional_params=None):
        """ Refreshes OAuth token.

        Args:
            additional_params (dict):  Any additional form parameters.

        Returns:
            OAuthToken: The refreshed OAuth token.

        """
        token = self._o_auth_api.refresh_token(
            self.build_basic_auth_header(),
            self._o_auth_token.refresh_token,
            ' '.join(self._o_auth_scopes) if self._o_auth_scopes else None,
            _optional_form_parameters=additional_params
        )
        if hasattr(token, 'expires_in'):
            current_utc_timestamp = AuthHelper.get_current_utc_timestamp()
            token.expiry = AuthHelper.get_token_expiry(current_utc_timestamp, token.expires_in)
        return token


class AuthorizationCodeAuthCredentials:

    @property
    def o_auth_client_id(self):
        return self._o_auth_client_id

    @property
    def o_auth_client_secret(self):
        return self._o_auth_client_secret

    @property
    def o_auth_redirect_uri(self):
        return self._o_auth_redirect_uri

    @property
    def o_auth_token(self):
        return self._o_auth_token

    @property
    def o_auth_scopes(self):
        return self._o_auth_scopes

    def __init__(self, o_auth_client_id, o_auth_client_secret,
                 o_auth_redirect_uri, o_auth_token=None, o_auth_scopes=None):
        if o_auth_client_id is None:
            raise ValueError('o_auth_client_id cannot be None')
        if o_auth_client_secret is None:
            raise ValueError('o_auth_client_secret cannot be None')
        if o_auth_redirect_uri is None:
            raise ValueError('o_auth_redirect_uri cannot be None')
        self._o_auth_client_id = o_auth_client_id
        self._o_auth_client_secret = o_auth_client_secret
        self._o_auth_redirect_uri = o_auth_redirect_uri
        self._o_auth_token = o_auth_token
        self._o_auth_scopes = o_auth_scopes

    def clone_with(self, o_auth_client_id=None, o_auth_client_secret=None,
                   o_auth_redirect_uri=None, o_auth_token=None,
                   o_auth_scopes=None):
        return AuthorizationCodeAuthCredentials(
            o_auth_client_id or self.o_auth_client_id,
            o_auth_client_secret or self.o_auth_client_secret,
            o_auth_redirect_uri or self.o_auth_redirect_uri,
            o_auth_token or self.o_auth_token,
            o_auth_scopes or self.o_auth_scopes)



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class AudioFeaturesObject(object):

    """Implementation of the 'AudioFeaturesObject' model.

    Attributes:
        acousticness (float): A confidence measure from 0.0 to 1.0 of whether
            the track is acoustic. 1.0 represents high confidence the track is
            acoustic.
        analysis_url (str): A URL to access the full audio analysis of this
            track. An access token is required to access this data.
        danceability (float): Danceability describes how suitable a track is
            for dancing based on a combination of musical elements including
            tempo, rhythm stability, beat strength, and overall regularity. A
            value of 0.0 is least danceable and 1.0 is most danceable.
        duration_ms (int): The duration of the track in milliseconds.
        energy (float): Energy is a measure from 0.0 to 1.0 and represents a
            perceptual measure of intensity and activity. Typically, energetic
            tracks feel fast, loud, and noisy. For example, death metal has
            high energy, while a Bach prelude scores low on the scale.
            Perceptual features contributing to this attribute include dynamic
            range, perceived loudness, timbre, onset rate, and general entropy.
        id (str): The Spotify ID for the track.
        instrumentalness (float): Predicts whether a track contains no vocals.
            "Ooh" and "aah" sounds are treated as instrumental in this
            context. Rap or spoken word tracks are clearly "vocal". The closer
            the instrumentalness value is to 1.0, the greater likelihood the
            track contains no vocal content. Values above 0.5 are intended to
            represent instrumental tracks, but confidence is higher as the
            value approaches 1.0.
        key (int): The key the track is in. Integers map to pitches using
            standard [Pitch Class
            notation](https://en.wikipedia.org/wiki/Pitch_class). E.g. 0 = C,
            1 = C♯/D♭, 2 = D, and so on. If no key was detected, the value is
            -1.
        liveness (float): Detects the presence of an audience in the
            recording. Higher liveness values represent an increased
            probability that the track was performed live. A value above 0.8
            provides strong likelihood that the track is live.
        loudness (float): The overall loudness of a track in decibels (dB).
            Loudness values are averaged across the entire track and are
            useful for comparing relative loudness of tracks. Loudness is the
            quality of a sound that is the primary psychological correlate of
            physical strength (amplitude). Values typically range between -60
            and 0 db.
        mode (int): Mode indicates the modality (major or minor) of a track,
            the type of scale from which its melodic content is derived. Major
            is represented by 1 and minor is 0.
        speechiness (float): Speechiness detects the presence of spoken words
            in a track. The more exclusively speech-like the recording (e.g.
            talk show, audio book, poetry), the closer to 1.0 the attribute
            value. Values above 0.66 describe tracks that are probably made
            entirely of spoken words. Values between 0.33 and 0.66 describe
            tracks that may contain both music and speech, either in sections
            or layered, including such cases as rap music. Values below 0.33
            most likely represent music and other non-speech-like tracks.
        tempo (float): The overall estimated tempo of a track in beats per
            minute (BPM). In musical terminology, tempo is the speed or pace
            of a given piece and derives directly from the average beat
            duration.
        time_signature (int): An estimated time signature. The time signature
            (meter) is a notational convention to specify how many beats are
            in each bar (or measure). The time signature ranges from 3 to 7
            indicating time signatures of "3/4", to "7/4".
        track_href (str): A link to the Web API endpoint providing full
            details of the track.
        mtype (Type5Enum): The object type.
        uri (str): The Spotify URI for the track.
        valence (float): A measure from 0.0 to 1.0 describing the musical
            positiveness conveyed by a track. Tracks with high valence sound
            more positive (e.g. happy, cheerful, euphoric), while tracks with
            low valence sound more negative (e.g. sad, depressed, angry).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "acousticness": 'acousticness',
        "analysis_url": 'analysis_url',
        "danceability": 'danceability',
        "duration_ms": 'duration_ms',
        "energy": 'energy',
        "id": 'id',
        "instrumentalness": 'instrumentalness',
        "key": 'key',
        "liveness": 'liveness',
        "loudness": 'loudness',
        "mode": 'mode',
        "speechiness": 'speechiness',
        "tempo": 'tempo',
        "time_signature": 'time_signature',
        "track_href": 'track_href',
        "mtype": 'type',
        "uri": 'uri',
        "valence": 'valence'
    }

    _optionals = [
        'acousticness',
        'analysis_url',
        'danceability',
        'duration_ms',
        'energy',
        'id',
        'instrumentalness',
        'key',
        'liveness',
        'loudness',
        'mode',
        'speechiness',
        'tempo',
        'time_signature',
        'track_href',
        'mtype',
        'uri',
        'valence',
    ]

    def __init__(self,
                 acousticness=APIHelper.SKIP,
                 analysis_url=APIHelper.SKIP,
                 danceability=APIHelper.SKIP,
                 duration_ms=APIHelper.SKIP,
                 energy=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 instrumentalness=APIHelper.SKIP,
                 key=APIHelper.SKIP,
                 liveness=APIHelper.SKIP,
                 loudness=APIHelper.SKIP,
                 mode=APIHelper.SKIP,
                 speechiness=APIHelper.SKIP,
                 tempo=APIHelper.SKIP,
                 time_signature=APIHelper.SKIP,
                 track_href=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 uri=APIHelper.SKIP,
                 valence=APIHelper.SKIP):
        """Constructor for the AudioFeaturesObject class"""

        # Initialize members of the class
        if acousticness is not APIHelper.SKIP:
            self.acousticness = acousticness 
        if analysis_url is not APIHelper.SKIP:
            self.analysis_url = analysis_url 
        if danceability is not APIHelper.SKIP:
            self.danceability = danceability 
        if duration_ms is not APIHelper.SKIP:
            self.duration_ms = duration_ms 
        if energy is not APIHelper.SKIP:
            self.energy = energy 
        if id is not APIHelper.SKIP:
            self.id = id 
        if instrumentalness is not APIHelper.SKIP:
            self.instrumentalness = instrumentalness 
        if key is not APIHelper.SKIP:
            self.key = key 
        if liveness is not APIHelper.SKIP:
            self.liveness = liveness 
        if loudness is not APIHelper.SKIP:
            self.loudness = loudness 
        if mode is not APIHelper.SKIP:
            self.mode = mode 
        if speechiness is not APIHelper.SKIP:
            self.speechiness = speechiness 
        if tempo is not APIHelper.SKIP:
            self.tempo = tempo 
        if time_signature is not APIHelper.SKIP:
            self.time_signature = time_signature 
        if track_href is not APIHelper.SKIP:
            self.track_href = track_href 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if uri is not APIHelper.SKIP:
            self.uri = uri 
        if valence is not APIHelper.SKIP:
            self.valence = valence 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        acousticness = dictionary.get("acousticness") if dictionary.get("acousticness") else APIHelper.SKIP
        analysis_url = dictionary.get("analysis_url") if dictionary.get("analysis_url") else APIHelper.SKIP
        danceability = dictionary.get("danceability") if dictionary.get("danceability") else APIHelper.SKIP
        duration_ms = dictionary.get("duration_ms") if dictionary.get("duration_ms") else APIHelper.SKIP
        energy = dictionary.get("energy") if dictionary.get("energy") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        instrumentalness = dictionary.get("instrumentalness") if dictionary.get("instrumentalness") else APIHelper.SKIP
        key = dictionary.get("key") if dictionary.get("key") else APIHelper.SKIP
        liveness = dictionary.get("liveness") if dictionary.get("liveness") else APIHelper.SKIP
        loudness = dictionary.get("loudness") if dictionary.get("loudness") else APIHelper.SKIP
        mode = dictionary.get("mode") if dictionary.get("mode") else APIHelper.SKIP
        speechiness = dictionary.get("speechiness") if dictionary.get("speechiness") else APIHelper.SKIP
        tempo = dictionary.get("tempo") if dictionary.get("tempo") else APIHelper.SKIP
        time_signature = dictionary.get("time_signature") if dictionary.get("time_signature") else APIHelper.SKIP
        track_href = dictionary.get("track_href") if dictionary.get("track_href") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        uri = dictionary.get("uri") if dictionary.get("uri") else APIHelper.SKIP
        valence = dictionary.get("valence") if dictionary.get("valence") else APIHelper.SKIP
        # Return an object of this model
        return cls(acousticness,
                   analysis_url,
                   danceability,
                   duration_ms,
                   energy,
                   id,
                   instrumentalness,
                   key,
                   liveness,
                   loudness,
                   mode,
                   speechiness,
                   tempo,
                   time_signature,
                   track_href,
                   mtype,
                   uri,
                   valence)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'acousticness={(self.acousticness if hasattr(self, "acousticness") else None)!r}, '
                f'analysis_url={(self.analysis_url if hasattr(self, "analysis_url") else None)!r}, '
                f'danceability={(self.danceability if hasattr(self, "danceability") else None)!r}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!r}, '
                f'energy={(self.energy if hasattr(self, "energy") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'instrumentalness={(self.instrumentalness if hasattr(self, "instrumentalness") else None)!r}, '
                f'key={(self.key if hasattr(self, "key") else None)!r}, '
                f'liveness={(self.liveness if hasattr(self, "liveness") else None)!r}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!r}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!r}, '
                f'speechiness={(self.speechiness if hasattr(self, "speechiness") else None)!r}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!r}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!r}, '
                f'track_href={(self.track_href if hasattr(self, "track_href") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!r}, '
                f'valence={(self.valence if hasattr(self, "valence") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'acousticness={(self.acousticness if hasattr(self, "acousticness") else None)!s}, '
                f'analysis_url={(self.analysis_url if hasattr(self, "analysis_url") else None)!s}, '
                f'danceability={(self.danceability if hasattr(self, "danceability") else None)!s}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!s}, '
                f'energy={(self.energy if hasattr(self, "energy") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'instrumentalness={(self.instrumentalness if hasattr(self, "instrumentalness") else None)!s}, '
                f'key={(self.key if hasattr(self, "key") else None)!s}, '
                f'liveness={(self.liveness if hasattr(self, "liveness") else None)!s}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!s}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!s}, '
                f'speechiness={(self.speechiness if hasattr(self, "speechiness") else None)!s}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!s}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!s}, '
                f'track_href={(self.track_href if hasattr(self, "track_href") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!s}, '
                f'valence={(self.valence if hasattr(self, "valence") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.followers_object import FollowersObject


class PlaylistOwnerObject(object):

    """Implementation of the 'PlaylistOwnerObject' model.

    Attributes:
        external_urls (ExternalUrlObject): Known public external URLs for this
            user.
        followers (FollowersObject): Information about the followers of this
            user.
        href (str): A link to the Web API endpoint for this user.
        id (str): The [Spotify user
            ID](/documentation/web-api/concepts/spotify-uris-ids) for this
            user.
        mtype (Type3Enum): The object type.
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for this
            user.
        display_name (str): The name displayed on the user's profile. `null`
            if not available.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "external_urls": 'external_urls',
        "followers": 'followers',
        "href": 'href',
        "id": 'id',
        "mtype": 'type',
        "uri": 'uri',
        "display_name": 'display_name'
    }

    _optionals = [
        'external_urls',
        'followers',
        'href',
        'id',
        'mtype',
        'uri',
        'display_name',
    ]

    _nullables = [
        'display_name',
    ]

    def __init__(self,
                 external_urls=APIHelper.SKIP,
                 followers=APIHelper.SKIP,
                 href=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 uri=APIHelper.SKIP,
                 display_name=APIHelper.SKIP):
        """Constructor for the PlaylistOwnerObject class"""

        # Initialize members of the class
        if external_urls is not APIHelper.SKIP:
            self.external_urls = external_urls 
        if followers is not APIHelper.SKIP:
            self.followers = followers 
        if href is not APIHelper.SKIP:
            self.href = href 
        if id is not APIHelper.SKIP:
            self.id = id 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if uri is not APIHelper.SKIP:
            self.uri = uri 
        if display_name is not APIHelper.SKIP:
            self.display_name = display_name 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if 'external_urls' in dictionary.keys() else APIHelper.SKIP
        followers = FollowersObject.from_dictionary(dictionary.get('followers')) if 'followers' in dictionary.keys() else APIHelper.SKIP
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        uri = dictionary.get("uri") if dictionary.get("uri") else APIHelper.SKIP
        display_name = dictionary.get("display_name") if "display_name" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(external_urls,
                   followers,
                   href,
                   id,
                   mtype,
                   uri,
                   display_name)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!r}, '
                f'followers={(self.followers if hasattr(self, "followers") else None)!r}, '
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!r}, '
                f'display_name={(self.display_name if hasattr(self, "display_name") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!s}, '
                f'followers={(self.followers if hasattr(self, "followers") else None)!s}, '
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!s}, '
                f'display_name={(self.display_name if hasattr(self, "display_name") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class ImageObject(object):

    """Implementation of the 'ImageObject' model.

    Attributes:
        url (str): The source URL of the image.
        height (int): The image height in pixels.
        width (int): The image width in pixels.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "url": 'url',
        "height": 'height',
        "width": 'width'
    }

    _nullables = [
        'height',
        'width',
    ]

    def __init__(self,
                 url=None,
                 height=None,
                 width=None):
        """Constructor for the ImageObject class"""

        # Initialize members of the class
        self.url = url 
        self.height = height 
        self.width = width 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        url = dictionary.get("url") if dictionary.get("url") else None
        height = dictionary.get("height") if dictionary.get("height") else None
        width = dictionary.get("width") if dictionary.get("width") else None
        # Return an object of this model
        return cls(url,
                   height,
                   width)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return APIHelper.is_valid_type(value=dictionary.url,
                                           type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.height,
                                            type_callable=lambda value: isinstance(value, int),
                                            is_value_nullable=True) \
                and APIHelper.is_valid_type(value=dictionary.width,
                                            type_callable=lambda value: isinstance(value, int),
                                            is_value_nullable=True)

        if not isinstance(dictionary, dict):
            return False

        return APIHelper.is_valid_type(value=dictionary.get('url'),
                                       type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('height'),
                                        type_callable=lambda value: isinstance(value, int),
                                        is_value_nullable=True) \
            and APIHelper.is_valid_type(value=dictionary.get('width'),
                                        type_callable=lambda value: isinstance(value, int),
                                        is_value_nullable=True)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'url={self.url!r}, '
                f'height={self.height!r}, '
                f'width={self.width!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'url={self.url!s}, '
                f'height={self.height!s}, '
                f'width={self.width!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.artist_object import ArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.cursor_object import CursorObject


class CursorPagingSimplifiedArtistObject(object):

    """Implementation of the 'CursorPagingSimplifiedArtistObject' model.

    Attributes:
        href (str): A link to the Web API endpoint returning the full result
            of the request.
        limit (int): The maximum number of items in the response (as set in
            the query or by default).
        next (str): URL to the next page of items. ( `null` if none)
        cursors (CursorObject): The cursors used to find the next set of items.
        total (int): The total number of items available to return.
        items (List[ArtistObject]): The model property of type
            List[ArtistObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "href": 'href',
        "limit": 'limit',
        "next": 'next',
        "cursors": 'cursors',
        "total": 'total',
        "items": 'items'
    }

    _optionals = [
        'href',
        'limit',
        'next',
        'cursors',
        'total',
        'items',
    ]

    def __init__(self,
                 href=APIHelper.SKIP,
                 limit=APIHelper.SKIP,
                 next=APIHelper.SKIP,
                 cursors=APIHelper.SKIP,
                 total=APIHelper.SKIP,
                 items=APIHelper.SKIP):
        """Constructor for the CursorPagingSimplifiedArtistObject class"""

        # Initialize members of the class
        if href is not APIHelper.SKIP:
            self.href = href 
        if limit is not APIHelper.SKIP:
            self.limit = limit 
        if next is not APIHelper.SKIP:
            self.next = next 
        if cursors is not APIHelper.SKIP:
            self.cursors = cursors 
        if total is not APIHelper.SKIP:
            self.total = total 
        if items is not APIHelper.SKIP:
            self.items = items 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        limit = dictionary.get("limit") if dictionary.get("limit") else APIHelper.SKIP
        next = dictionary.get("next") if dictionary.get("next") else APIHelper.SKIP
        cursors = CursorObject.from_dictionary(dictionary.get('cursors')) if 'cursors' in dictionary.keys() else APIHelper.SKIP
        total = dictionary.get("total") if dictionary.get("total") else APIHelper.SKIP
        items = None
        if dictionary.get('items') is not None:
            items = [ArtistObject.from_dictionary(x) for x in dictionary.get('items')]
        else:
            items = APIHelper.SKIP
        # Return an object of this model
        return cls(href,
                   limit,
                   next,
                   cursors,
                   total,
                   items)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'limit={(self.limit if hasattr(self, "limit") else None)!r}, '
                f'next={(self.next if hasattr(self, "next") else None)!r}, '
                f'cursors={(self.cursors if hasattr(self, "cursors") else None)!r}, '
                f'total={(self.total if hasattr(self, "total") else None)!r}, '
                f'items={(self.items if hasattr(self, "items") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'limit={(self.limit if hasattr(self, "limit") else None)!s}, '
                f'next={(self.next if hasattr(self, "next") else None)!s}, '
                f'cursors={(self.cursors if hasattr(self, "cursors") else None)!s}, '
                f'total={(self.total if hasattr(self, "total") else None)!s}, '
                f'items={(self.items if hasattr(self, "items") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paged_categories import PagedCategories
from spotifywebapiwithfixesandimprovementsfromsonallux.models.category_object import CategoryObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class CategoriesController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(CategoriesController, self).__init__(config)

    def get_categories(self,
                       locale=None,
                       limit=20,
                       offset=0):
        """Does a GET request to /browse/categories.

        Get a list of categories used to tag items in Spotify (on, for
        example, the Spotify player’s “Browse” tab).

        Args:
            locale (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagedCategories: Response from the API. A paged set of categories

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/browse/categories')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('locale')
                         .value(locale))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagedCategories.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_a_category(self,
                       category_id,
                       locale=None):
        """Does a GET request to /browse/categories/{category_id}.

        Get a single category used to tag items in Spotify (on, for example,
        the Spotify player’s “Browse” tab).

        Args:
            category_id (str): The request template parameter.
            locale (str, optional): The request query parameter.

        Returns:
            CategoryObject: Response from the API. A category

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/browse/categories/{category_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('category_id')
                            .value(category_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('locale')
                         .value(locale))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CategoryObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class ExternalIdObject(object):

    """Implementation of the 'ExternalIdObject' model.

    Attributes:
        isrc (str): [International Standard Recording
            Code](http://en.wikipedia.org/wiki/International_Standard_Recording
            _Code)
        ean (str): [International Article
            Number](http://en.wikipedia.org/wiki/International_Article_Number_%
            28EAN%29)
        upc (str): [Universal Product
            Code](http://en.wikipedia.org/wiki/Universal_Product_Code)

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "isrc": 'isrc',
        "ean": 'ean',
        "upc": 'upc'
    }

    _optionals = [
        'isrc',
        'ean',
        'upc',
    ]

    def __init__(self,
                 isrc=APIHelper.SKIP,
                 ean=APIHelper.SKIP,
                 upc=APIHelper.SKIP):
        """Constructor for the ExternalIdObject class"""

        # Initialize members of the class
        if isrc is not APIHelper.SKIP:
            self.isrc = isrc 
        if ean is not APIHelper.SKIP:
            self.ean = ean 
        if upc is not APIHelper.SKIP:
            self.upc = upc 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        isrc = dictionary.get("isrc") if dictionary.get("isrc") else APIHelper.SKIP
        ean = dictionary.get("ean") if dictionary.get("ean") else APIHelper.SKIP
        upc = dictionary.get("upc") if dictionary.get("upc") else APIHelper.SKIP
        # Return an object of this model
        return cls(isrc,
                   ean,
                   upc)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'isrc={(self.isrc if hasattr(self, "isrc") else None)!r}, '
                f'ean={(self.ean if hasattr(self, "ean") else None)!r}, '
                f'upc={(self.upc if hasattr(self, "upc") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'isrc={(self.isrc if hasattr(self, "isrc") else None)!s}, '
                f'ean={(self.ean if hasattr(self, "ean") else None)!s}, '
                f'upc={(self.upc if hasattr(self, "upc") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_genres import ManyGenres
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class GenresController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(GenresController, self).__init__(config)

    def get_recommendation_genres(self):
        """Does a GET request to /recommendations/available-genre-seeds.

        Retrieve a list of available genres seed parameter values for
        [recommendations](/documentation/web-api/reference/get-recommendations)
        .

        Returns:
            ManyGenres: Response from the API. A set of genres

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/recommendations/available-genre-seeds')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyGenres.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.playlist_user_object import PlaylistUserObject


class PlaylistTrackObject(object):

    """Implementation of the 'PlaylistTrackObject' model.

    Attributes:
        added_at (datetime): The date and time the track or episode was added.
            _**Note**: some very old playlists may return `null` in this
            field._
        added_by (PlaylistUserObject): The Spotify user who added the track or
            episode. _**Note**: some very old playlists may return `null` in
            this field._
        is_local (bool): Whether this track or episode is a [local
            file](/documentation/web-api/concepts/playlists/#local-files) or
            not.
        track (TrackObject | EpisodeObject | None): Information about the
            track or episode.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "added_at": 'added_at',
        "added_by": 'added_by',
        "is_local": 'is_local',
        "track": 'track'
    }

    _optionals = [
        'added_at',
        'added_by',
        'is_local',
        'track',
    ]

    def __init__(self,
                 added_at=APIHelper.SKIP,
                 added_by=APIHelper.SKIP,
                 is_local=APIHelper.SKIP,
                 track=APIHelper.SKIP):
        """Constructor for the PlaylistTrackObject class"""

        # Initialize members of the class
        if added_at is not APIHelper.SKIP:
            self.added_at = APIHelper.apply_datetime_converter(added_at, APIHelper.RFC3339DateTime) if added_at else None 
        if added_by is not APIHelper.SKIP:
            self.added_by = added_by 
        if is_local is not APIHelper.SKIP:
            self.is_local = is_local 
        if track is not APIHelper.SKIP:
            self.track = track 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        from spotifywebapiwithfixesandimprovementsfromsonallux.utilities.union_type_lookup import UnionTypeLookUp

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        added_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("added_at")).datetime if dictionary.get("added_at") else APIHelper.SKIP
        added_by = PlaylistUserObject.from_dictionary(dictionary.get('added_by')) if 'added_by' in dictionary.keys() else APIHelper.SKIP
        is_local = dictionary.get("is_local") if "is_local" in dictionary.keys() else APIHelper.SKIP
        track = APIHelper.deserialize_union_type(UnionTypeLookUp.get('PlaylistTrackObjectTrack'), dictionary.get('track'), False) if dictionary.get('track') is not None else APIHelper.SKIP
        # Return an object of this model
        return cls(added_at,
                   added_by,
                   is_local,
                   track)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'added_at={(self.added_at if hasattr(self, "added_at") else None)!r}, '
                f'added_by={(self.added_by if hasattr(self, "added_by") else None)!r}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!r}, '
                f'track={(self.track if hasattr(self, "track") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'added_at={(self.added_at if hasattr(self, "added_at") else None)!s}, '
                f'added_by={(self.added_by if hasattr(self, "added_by") else None)!s}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!s}, '
                f'track={(self.track if hasattr(self, "track") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.categories import Categories


class PagedCategories(object):

    """Implementation of the 'PagedCategories' model.

    Attributes:
        categories (Categories): The model property of type Categories.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "categories": 'categories'
    }

    def __init__(self,
                 categories=None):
        """Constructor for the PagedCategories class"""

        # Initialize members of the class
        self.categories = categories 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        categories = Categories.from_dictionary(dictionary.get('categories')) if dictionary.get('categories') else None
        # Return an object of this model
        return cls(categories)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'categories={self.categories!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'categories={self.categories!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.chapter_object import ChapterObject


class ManyChapters(object):

    """Implementation of the 'ManyChapters' model.

    Attributes:
        chapters (List[ChapterObject]): The model property of type
            List[ChapterObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "chapters": 'chapters'
    }

    def __init__(self,
                 chapters=None):
        """Constructor for the ManyChapters class"""

        # Initialize members of the class
        self.chapters = chapters 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        chapters = None
        if dictionary.get('chapters') is not None:
            chapters = [ChapterObject.from_dictionary(x) for x in dictionary.get('chapters')]
        # Return an object of this model
        return cls(chapters)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'chapters={self.chapters!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'chapters={self.chapters!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.artist_object import ArtistObject


class ManyArtists(object):

    """Implementation of the 'ManyArtists' model.

    Attributes:
        artists (List[ArtistObject]): The model property of type
            List[ArtistObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "artists": 'artists'
    }

    def __init__(self,
                 artists=None):
        """Constructor for the ManyArtists class"""

        # Initialize members of the class
        self.artists = artists 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        artists = None
        if dictionary.get('artists') is not None:
            artists = [ArtistObject.from_dictionary(x) for x in dictionary.get('artists')]
        # Return an object of this model
        return cls(artists)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'artists={self.artists!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'artists={self.artists!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.context_object import ContextObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.device_object import DeviceObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.disallows_object import DisallowsObject


class CurrentlyPlayingContextObject(object):

    """Implementation of the 'CurrentlyPlayingContextObject' model.

    Attributes:
        device (DeviceObject): The device that is currently active.
        repeat_state (str): off, track, context
        shuffle_state (bool): If shuffle is on or off.
        context (ContextObject): A Context Object. Can be `null`.
        timestamp (int): Unix Millisecond Timestamp when data was fetched.
        progress_ms (int): Progress into the currently playing track or
            episode. Can be `null`.
        is_playing (bool): If something is currently playing, return `true`.
        item (TrackObject | EpisodeObject | None): The currently playing track
            or episode. Can be `null`.
        currently_playing_type (str): The object type of the currently playing
            item. Can be one of `track`, `episode`, `ad` or `unknown`.
        actions (DisallowsObject): Allows to update the user interface based
            on which playback actions are available within the current context.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device": 'device',
        "repeat_state": 'repeat_state',
        "shuffle_state": 'shuffle_state',
        "context": 'context',
        "timestamp": 'timestamp',
        "progress_ms": 'progress_ms',
        "is_playing": 'is_playing',
        "item": 'item',
        "currently_playing_type": 'currently_playing_type',
        "actions": 'actions'
    }

    _optionals = [
        'device',
        'repeat_state',
        'shuffle_state',
        'context',
        'timestamp',
        'progress_ms',
        'is_playing',
        'item',
        'currently_playing_type',
        'actions',
    ]

    def __init__(self,
                 device=APIHelper.SKIP,
                 repeat_state=APIHelper.SKIP,
                 shuffle_state=APIHelper.SKIP,
                 context=APIHelper.SKIP,
                 timestamp=APIHelper.SKIP,
                 progress_ms=APIHelper.SKIP,
                 is_playing=APIHelper.SKIP,
                 item=APIHelper.SKIP,
                 currently_playing_type=APIHelper.SKIP,
                 actions=APIHelper.SKIP):
        """Constructor for the CurrentlyPlayingContextObject class"""

        # Initialize members of the class
        if device is not APIHelper.SKIP:
            self.device = device 
        if repeat_state is not APIHelper.SKIP:
            self.repeat_state = repeat_state 
        if shuffle_state is not APIHelper.SKIP:
            self.shuffle_state = shuffle_state 
        if context is not APIHelper.SKIP:
            self.context = context 
        if timestamp is not APIHelper.SKIP:
            self.timestamp = timestamp 
        if progress_ms is not APIHelper.SKIP:
            self.progress_ms = progress_ms 
        if is_playing is not APIHelper.SKIP:
            self.is_playing = is_playing 
        if item is not APIHelper.SKIP:
            self.item = item 
        if currently_playing_type is not APIHelper.SKIP:
            self.currently_playing_type = currently_playing_type 
        if actions is not APIHelper.SKIP:
            self.actions = actions 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        from spotifywebapiwithfixesandimprovementsfromsonallux.utilities.union_type_lookup import UnionTypeLookUp

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        device = DeviceObject.from_dictionary(dictionary.get('device')) if 'device' in dictionary.keys() else APIHelper.SKIP
        repeat_state = dictionary.get("repeat_state") if dictionary.get("repeat_state") else APIHelper.SKIP
        shuffle_state = dictionary.get("shuffle_state") if "shuffle_state" in dictionary.keys() else APIHelper.SKIP
        context = ContextObject.from_dictionary(dictionary.get('context')) if 'context' in dictionary.keys() else APIHelper.SKIP
        timestamp = dictionary.get("timestamp") if dictionary.get("timestamp") else APIHelper.SKIP
        progress_ms = dictionary.get("progress_ms") if dictionary.get("progress_ms") else APIHelper.SKIP
        is_playing = dictionary.get("is_playing") if "is_playing" in dictionary.keys() else APIHelper.SKIP
        item = APIHelper.deserialize_union_type(UnionTypeLookUp.get('CurrentlyPlayingContextObjectItem'), dictionary.get('item'), False) if dictionary.get('item') is not None else APIHelper.SKIP
        currently_playing_type = dictionary.get("currently_playing_type") if dictionary.get("currently_playing_type") else APIHelper.SKIP
        actions = DisallowsObject.from_dictionary(dictionary.get('actions')) if 'actions' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(device,
                   repeat_state,
                   shuffle_state,
                   context,
                   timestamp,
                   progress_ms,
                   is_playing,
                   item,
                   currently_playing_type,
                   actions)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'device={(self.device if hasattr(self, "device") else None)!r}, '
                f'repeat_state={(self.repeat_state if hasattr(self, "repeat_state") else None)!r}, '
                f'shuffle_state={(self.shuffle_state if hasattr(self, "shuffle_state") else None)!r}, '
                f'context={(self.context if hasattr(self, "context") else None)!r}, '
                f'timestamp={(self.timestamp if hasattr(self, "timestamp") else None)!r}, '
                f'progress_ms={(self.progress_ms if hasattr(self, "progress_ms") else None)!r}, '
                f'is_playing={(self.is_playing if hasattr(self, "is_playing") else None)!r}, '
                f'item={(self.item if hasattr(self, "item") else None)!r}, '
                f'currently_playing_type={(self.currently_playing_type if hasattr(self, "currently_playing_type") else None)!r}, '
                f'actions={(self.actions if hasattr(self, "actions") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'device={(self.device if hasattr(self, "device") else None)!s}, '
                f'repeat_state={(self.repeat_state if hasattr(self, "repeat_state") else None)!s}, '
                f'shuffle_state={(self.shuffle_state if hasattr(self, "shuffle_state") else None)!s}, '
                f'context={(self.context if hasattr(self, "context") else None)!s}, '
                f'timestamp={(self.timestamp if hasattr(self, "timestamp") else None)!s}, '
                f'progress_ms={(self.progress_ms if hasattr(self, "progress_ms") else None)!s}, '
                f'is_playing={(self.is_playing if hasattr(self, "is_playing") else None)!s}, '
                f'item={(self.item if hasattr(self, "item") else None)!s}, '
                f'currently_playing_type={(self.currently_playing_type if hasattr(self, "currently_playing_type") else None)!s}, '
                f'actions={(self.actions if hasattr(self, "actions") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class PlaylistsTracksRequest(object):

    """Implementation of the 'Playlists Tracks Request' model.

    Attributes:
        uris (List[str]): A JSON array of the [Spotify
            URIs](/documentation/web-api/concepts/spotify-uris-ids) to add.
            For example: `{"uris":
            ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98M
            SxVHPZCA6M", "spotify:episode:512ojhOuo1ktJprKbVcKyQ"]}`<br/>A
            maximum of 100 items can be added in one request. _**Note**: if
            the `uris` parameter is present in the query string, any URIs
            listed here in the body will be ignored._
        position (int): The position to insert the items, a zero-based index.
            For example, to insert the items in the first position:
            `position=0` ; to insert the items in the third position:
            `position=2`. If omitted, the items will be appended to the
            playlist. Items are added in the order they appear in the uris
            array. For example: `{"uris":
            ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98M
            SxVHPZCA6M"], "position": 3}`

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "uris": 'uris',
        "position": 'position'
    }

    _optionals = [
        'uris',
        'position',
    ]

    def __init__(self,
                 uris=APIHelper.SKIP,
                 position=APIHelper.SKIP):
        """Constructor for the PlaylistsTracksRequest class"""

        # Initialize members of the class
        if uris is not APIHelper.SKIP:
            self.uris = uris 
        if position is not APIHelper.SKIP:
            self.position = position 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        uris = dictionary.get("uris") if dictionary.get("uris") else APIHelper.SKIP
        position = dictionary.get("position") if dictionary.get("position") else APIHelper.SKIP
        # Return an object of this model
        return cls(uris,
                   position)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'uris={(self.uris if hasattr(self, "uris") else None)!r}, '
                f'position={(self.position if hasattr(self, "position") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'uris={(self.uris if hasattr(self, "uris") else None)!s}, '
                f'position={(self.position if hasattr(self, "position") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.context_object import ContextObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_object import TrackObject


class PlayHistoryObject(object):

    """Implementation of the 'PlayHistoryObject' model.

    Attributes:
        track (TrackObject): The track the user listened to.
        played_at (datetime): The date and time the track was played.
        context (ContextObject): The context the track was played from.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "track": 'track',
        "played_at": 'played_at',
        "context": 'context'
    }

    _optionals = [
        'track',
        'played_at',
        'context',
    ]

    def __init__(self,
                 track=APIHelper.SKIP,
                 played_at=APIHelper.SKIP,
                 context=APIHelper.SKIP):
        """Constructor for the PlayHistoryObject class"""

        # Initialize members of the class
        if track is not APIHelper.SKIP:
            self.track = track 
        if played_at is not APIHelper.SKIP:
            self.played_at = APIHelper.apply_datetime_converter(played_at, APIHelper.RFC3339DateTime) if played_at else None 
        if context is not APIHelper.SKIP:
            self.context = context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        track = TrackObject.from_dictionary(dictionary.get('track')) if 'track' in dictionary.keys() else APIHelper.SKIP
        played_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("played_at")).datetime if dictionary.get("played_at") else APIHelper.SKIP
        context = ContextObject.from_dictionary(dictionary.get('context')) if 'context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(track,
                   played_at,
                   context)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'track={(self.track if hasattr(self, "track") else None)!r}, '
                f'played_at={(self.played_at if hasattr(self, "played_at") else None)!r}, '
                f'context={(self.context if hasattr(self, "context") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'track={(self.track if hasattr(self, "track") else None)!s}, '
                f'played_at={(self.played_at if hasattr(self, "played_at") else None)!s}, '
                f'context={(self.context if hasattr(self, "context") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_playlist_object import PagingPlaylistObject


class PagingFeaturedPlaylistObject(object):

    """Implementation of the 'PagingFeaturedPlaylistObject' model.

    Attributes:
        message (str): The localized message of a playlist.
        playlists (PagingPlaylistObject): The model property of type
            PagingPlaylistObject.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "message": 'message',
        "playlists": 'playlists'
    }

    _optionals = [
        'message',
        'playlists',
    ]

    def __init__(self,
                 message=APIHelper.SKIP,
                 playlists=APIHelper.SKIP):
        """Constructor for the PagingFeaturedPlaylistObject class"""

        # Initialize members of the class
        if message is not APIHelper.SKIP:
            self.message = message 
        if playlists is not APIHelper.SKIP:
            self.playlists = playlists 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        message = dictionary.get("message") if dictionary.get("message") else APIHelper.SKIP
        playlists = PagingPlaylistObject.from_dictionary(dictionary.get('playlists')) if 'playlists' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(message,
                   playlists)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'message={(self.message if hasattr(self, "message") else None)!r}, '
                f'playlists={(self.playlists if hasattr(self, "playlists") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'message={(self.message if hasattr(self, "message") else None)!s}, '
                f'playlists={(self.playlists if hasattr(self, "playlists") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class MePlayerRequest(object):

    """Implementation of the 'Me Player Request' model.

    Attributes:
        device_ids (List[str]): A JSON array containing the ID of the device
            on which playback should be started/transferred.<br/>For
            example:`{device_ids:["74ASZWbe4lXaubB36ztrGX"]}`<br/>_**Note**:
            Although an array is accepted, only a single device_id is
            currently supported. Supplying more than one will return `400 Bad
            Request`_
        play (bool): **true**: ensure playback happens on new
            device.<br/>**false** or not provided: keep the current playback
            state.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_ids": 'device_ids',
        "play": 'play'
    }

    _optionals = [
        'play',
    ]

    def __init__(self,
                 device_ids=None,
                 play=APIHelper.SKIP):
        """Constructor for the MePlayerRequest class"""

        # Initialize members of the class
        self.device_ids = device_ids 
        if play is not APIHelper.SKIP:
            self.play = play 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        device_ids = dictionary.get("device_ids") if dictionary.get("device_ids") else None
        play = dictionary.get("play") if "play" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(device_ids,
                   play)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'device_ids={self.device_ids!r}, '
                f'play={(self.play if hasattr(self, "play") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'device_ids={self.device_ids!s}, '
                f'play={(self.play if hasattr(self, "play") else None)!s})')



__all__ = [
    'linked_track_object',
    'track_restriction_object',
    'album_restriction_object',
    'episode_restriction_object',
    'chapter_restriction_object',
    'artist_object',
    'simplified_artist_object',
    'play_history_object',
    'playlist_track_object',
    'queue_object',
    'currently_playing_context_object',
    'disallows_object',
    'error_object',
    'private_user_object',
    'public_user_object',
    'audio_analysis_object',
    'time_interval_object',
    'section_object',
    'segment_object',
    'audio_features_object',
    'simplified_track_object',
    'device_object',
    'cursor_object',
    'cursor_paging_object',
    'cursor_paging_play_history_object',
    'cursor_paging_simplified_artist_object',
    'paging_object',
    'paging_playlist_object',
    'paging_featured_playlist_object',
    'paging_artist_discography_album_object',
    'paging_simplified_album_object',
    'paging_saved_album_object',
    'paging_simplified_track_object',
    'paging_saved_track_object',
    'paging_track_object',
    'paging_playlist_track_object',
    'paging_simplified_show_object',
    'paging_saved_show_object',
    'paging_simplified_episode_object',
    'paging_saved_episode_object',
    'paging_simplified_audiobook_object',
    'paging_artist_object',
    'paging_simplified_chapter_object',
    'recommendations_object',
    'recommendation_seed_object',
    'saved_album_object',
    'saved_track_object',
    'saved_episode_object',
    'saved_show_object',
    'playlist_object',
    'simplified_playlist_object',
    'playlist_tracks_ref_object',
    'playlist_user_object',
    'playlist_owner_object',
    'category_object',
    'track_object',
    'episode_object',
    'episode_base',
    'resume_point_object',
    'show_base',
    'show_object',
    'audiobook_base',
    'audiobook_object',
    'album_base',
    'simplified_album_object',
    'artist_discography_album_object',
    'chapter_object',
    'chapter_base',
    'album_object',
    'context_object',
    'copyright_object',
    'author_object',
    'narrator_object',
    'external_id_object',
    'external_url_object',
    'followers_object',
    'image_object',
    'explicit_content_settings_object',
    'currently_playing_object',
    'saved_audiobook_object',
    'paging_saved_audiobook_object',
    'categories',
    'cursor_paged_artists',
    'many_albums',
    'many_artists',
    'many_audiobooks',
    'many_audio_features',
    'many_chapters',
    'many_devices',
    'many_episodes',
    'many_genres',
    'many_simplified_shows',
    'many_tracks',
    'markets',
    'me_albums_request',
    'me_episodes_request',
    'me_episodes_request_1',
    'me_following_request',
    'me_following_request_1',
    'me_player_play_request',
    'me_player_request',
    'me_shows_request',
    'me_tracks_request',
    'me_tracks_request_1',
    'meta',
    'paged_albums',
    'paged_categories',
    'playlists_followers_request',
    'playlists_request',
    'playlists_tracks_request',
    'playlists_tracks_request_1',
    'playlists_tracks_request_2',
    'playlist_snapshot_id',
    'search_items',
    'track',
    'track_1',
    'users_playlists_request',
    'o_auth_token',
    'album_group_enum',
    'album_type_enum',
    'include_external_enum',
    'item_type_enum',
    'item_type_1_enum',
    'item_type_2_enum',
    'item_type_3_enum',
    'mode_enum',
    'reason_enum',
    'release_date_precision_enum',
    'type_enum',
    'type_2_enum',
    'type_3_enum',
    'type_5_enum',
    'o_auth_provider_error_enum',
    'o_auth_scope_enum',
]



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_object import TrackObject


class SavedTrackObject(object):

    """Implementation of the 'SavedTrackObject' model.

    Attributes:
        added_at (datetime): The date and time the track was saved. Timestamps
            are returned in ISO 8601 format as Coordinated Universal Time
            (UTC) with a zero offset: YYYY-MM-DDTHH:MM:SSZ. If the time is
            imprecise (for example, the date/time of an album release), an
            additional field indicates the precision; see for example,
            release_date in an album object.
        track (TrackObject): Information about the track.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "added_at": 'added_at',
        "track": 'track'
    }

    _optionals = [
        'added_at',
        'track',
    ]

    def __init__(self,
                 added_at=APIHelper.SKIP,
                 track=APIHelper.SKIP):
        """Constructor for the SavedTrackObject class"""

        # Initialize members of the class
        if added_at is not APIHelper.SKIP:
            self.added_at = APIHelper.apply_datetime_converter(added_at, APIHelper.RFC3339DateTime) if added_at else None 
        if track is not APIHelper.SKIP:
            self.track = track 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        added_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("added_at")).datetime if dictionary.get("added_at") else APIHelper.SKIP
        track = TrackObject.from_dictionary(dictionary.get('track')) if 'track' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(added_at,
                   track)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'added_at={(self.added_at if hasattr(self, "added_at") else None)!r}, '
                f'track={(self.track if hasattr(self, "track") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'added_at={(self.added_at if hasattr(self, "added_at") else None)!s}, '
                f'track={(self.track if hasattr(self, "track") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class MeTracksRequest1(object):

    """Implementation of the 'Me Tracks Request1' model.

    Attributes:
        ids (List[str]): A JSON array of the [Spotify
            IDs](/documentation/web-api/concepts/spotify-uris-ids). For
            example: `["4iV5W9uYEdYUVa79Axb7Rh",
            "1301WleyT98MSxVHPZCA6M"]`<br/>A maximum of 50 items can be
            specified in one request. _**Note**: if the `ids` parameter is
            present in the query string, any IDs listed here in the body will
            be ignored._

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "ids": 'ids'
    }

    _optionals = [
        'ids',
    ]

    def __init__(self,
                 ids=APIHelper.SKIP):
        """Constructor for the MeTracksRequest1 class"""

        # Initialize members of the class
        if ids is not APIHelper.SKIP:
            self.ids = ids 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        ids = dictionary.get("ids") if dictionary.get("ids") else APIHelper.SKIP
        # Return an object of this model
        return cls(ids)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'ids={(self.ids if hasattr(self, "ids") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'ids={(self.ids if hasattr(self, "ids") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.copyright_object import CopyrightObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject


class ShowBase(object):

    """Implementation of the 'ShowBase' model.

    Attributes:
        available_markets (List[str]): A list of the countries in which the
            show can be played, identified by their [ISO 3166-1
            alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
        copyrights (List[CopyrightObject]): The copyright statements of the
            show.
        description (str): A description of the show. HTML tags are stripped
            away from this field, use `html_description` field in case HTML
            tags are needed.
        html_description (str): A description of the show. This field may
            contain HTML tags.
        explicit (bool): Whether or not the show has explicit content (true =
            yes it does; false = no it does not OR unknown).
        external_urls (ExternalUrlObject): External URLs for this show.
        href (str): A link to the Web API endpoint providing full details of
            the show.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the show.
        images (List[ImageObject]): The cover art for the show in various
            sizes, widest first.
        is_externally_hosted (bool): True if all of the shows episodes are
            hosted outside of Spotify's CDN. This field might be `null` in
            some cases.
        languages (List[str]): A list of the languages used in the show,
            identified by their [ISO
            639](https://en.wikipedia.org/wiki/ISO_639) code.
        media_type (str): The media type of the show.
        name (str): The name of the episode.
        publisher (str): The publisher of the show.
        mtype (str): The object type.
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            show.
        total_episodes (int): The total number of episodes in the show.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "available_markets": 'available_markets',
        "copyrights": 'copyrights',
        "description": 'description',
        "html_description": 'html_description',
        "explicit": 'explicit',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "images": 'images',
        "is_externally_hosted": 'is_externally_hosted',
        "languages": 'languages',
        "media_type": 'media_type',
        "name": 'name',
        "publisher": 'publisher',
        "mtype": 'type',
        "uri": 'uri',
        "total_episodes": 'total_episodes'
    }

    def __init__(self,
                 available_markets=None,
                 copyrights=None,
                 description=None,
                 html_description=None,
                 explicit=None,
                 external_urls=None,
                 href=None,
                 id=None,
                 images=None,
                 is_externally_hosted=None,
                 languages=None,
                 media_type=None,
                 name=None,
                 publisher=None,
                 uri=None,
                 total_episodes=None):
        """Constructor for the ShowBase class"""

        # Initialize members of the class
        self.available_markets = available_markets 
        self.copyrights = copyrights 
        self.description = description 
        self.html_description = html_description 
        self.explicit = explicit 
        self.external_urls = external_urls 
        self.href = href 
        self.id = id 
        self.images = images 
        self.is_externally_hosted = is_externally_hosted 
        self.languages = languages 
        self.media_type = media_type 
        self.name = name 
        self.publisher = publisher 
        self.mtype = 'show' 
        self.uri = uri 
        self.total_episodes = total_episodes 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        available_markets = dictionary.get("available_markets") if dictionary.get("available_markets") else None
        copyrights = None
        if dictionary.get('copyrights') is not None:
            copyrights = [CopyrightObject.from_dictionary(x) for x in dictionary.get('copyrights')]
        description = dictionary.get("description") if dictionary.get("description") else None
        html_description = dictionary.get("html_description") if dictionary.get("html_description") else None
        explicit = dictionary.get("explicit") if "explicit" in dictionary.keys() else None
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if dictionary.get('external_urls') else None
        href = dictionary.get("href") if dictionary.get("href") else None
        id = dictionary.get("id") if dictionary.get("id") else None
        images = None
        if dictionary.get('images') is not None:
            images = [ImageObject.from_dictionary(x) for x in dictionary.get('images')]
        is_externally_hosted = dictionary.get("is_externally_hosted") if "is_externally_hosted" in dictionary.keys() else None
        languages = dictionary.get("languages") if dictionary.get("languages") else None
        media_type = dictionary.get("media_type") if dictionary.get("media_type") else None
        name = dictionary.get("name") if dictionary.get("name") else None
        publisher = dictionary.get("publisher") if dictionary.get("publisher") else None
        uri = dictionary.get("uri") if dictionary.get("uri") else None
        total_episodes = dictionary.get("total_episodes") if dictionary.get("total_episodes") else None
        # Return an object of this model
        return cls(available_markets,
                   copyrights,
                   description,
                   html_description,
                   explicit,
                   external_urls,
                   href,
                   id,
                   images,
                   is_externally_hosted,
                   languages,
                   media_type,
                   name,
                   publisher,
                   uri,
                   total_episodes)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return APIHelper.is_valid_type(value=dictionary.available_markets,
                                           type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.copyrights,
                                            type_callable=lambda value: CopyrightObject.validate(value),
                                            is_model_dict=True,
                                            is_inner_model_dict=True) \
                and APIHelper.is_valid_type(value=dictionary.description,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.html_description,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.explicit,
                                            type_callable=lambda value: isinstance(value, bool)) \
                and APIHelper.is_valid_type(value=dictionary.external_urls,
                                            type_callable=lambda value: ExternalUrlObject.validate(value),
                                            is_model_dict=True) \
                and APIHelper.is_valid_type(value=dictionary.href,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.id,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.images,
                                            type_callable=lambda value: ImageObject.validate(value),
                                            is_model_dict=True,
                                            is_inner_model_dict=True) \
                and APIHelper.is_valid_type(value=dictionary.is_externally_hosted,
                                            type_callable=lambda value: isinstance(value, bool)) \
                and APIHelper.is_valid_type(value=dictionary.languages,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.media_type,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.name,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.publisher,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.mtype,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.uri,
                                            type_callable=lambda value: isinstance(value, str)) \
                and APIHelper.is_valid_type(value=dictionary.total_episodes,
                                            type_callable=lambda value: isinstance(value, int))

        if not isinstance(dictionary, dict):
            return False

        return APIHelper.is_valid_type(value=dictionary.get('available_markets'),
                                       type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('copyrights'),
                                        type_callable=lambda value: CopyrightObject.validate(value),
                                        is_model_dict=True,
                                        is_inner_model_dict=True) \
            and APIHelper.is_valid_type(value=dictionary.get('description'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('html_description'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('explicit'),
                                        type_callable=lambda value: isinstance(value, bool)) \
            and APIHelper.is_valid_type(value=dictionary.get('external_urls'),
                                        type_callable=lambda value: ExternalUrlObject.validate(value),
                                        is_model_dict=True) \
            and APIHelper.is_valid_type(value=dictionary.get('href'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('id'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('images'),
                                        type_callable=lambda value: ImageObject.validate(value),
                                        is_model_dict=True,
                                        is_inner_model_dict=True) \
            and APIHelper.is_valid_type(value=dictionary.get('is_externally_hosted'),
                                        type_callable=lambda value: isinstance(value, bool)) \
            and APIHelper.is_valid_type(value=dictionary.get('languages'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('media_type'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('name'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('publisher'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('type'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('uri'),
                                        type_callable=lambda value: isinstance(value, str)) \
            and APIHelper.is_valid_type(value=dictionary.get('total_episodes'),
                                        type_callable=lambda value: isinstance(value, int))

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'available_markets={self.available_markets!r}, '
                f'copyrights={self.copyrights!r}, '
                f'description={self.description!r}, '
                f'html_description={self.html_description!r}, '
                f'explicit={self.explicit!r}, '
                f'external_urls={self.external_urls!r}, '
                f'href={self.href!r}, '
                f'id={self.id!r}, '
                f'images={self.images!r}, '
                f'is_externally_hosted={self.is_externally_hosted!r}, '
                f'languages={self.languages!r}, '
                f'media_type={self.media_type!r}, '
                f'name={self.name!r}, '
                f'publisher={self.publisher!r}, '
                f'mtype={self.mtype!r}, '
                f'uri={self.uri!r}, '
                f'total_episodes={self.total_episodes!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'available_markets={self.available_markets!s}, '
                f'copyrights={self.copyrights!s}, '
                f'description={self.description!s}, '
                f'html_description={self.html_description!s}, '
                f'explicit={self.explicit!s}, '
                f'external_urls={self.external_urls!s}, '
                f'href={self.href!s}, '
                f'id={self.id!s}, '
                f'images={self.images!s}, '
                f'is_externally_hosted={self.is_externally_hosted!s}, '
                f'languages={self.languages!s}, '
                f'media_type={self.media_type!s}, '
                f'name={self.name!s}, '
                f'publisher={self.publisher!s}, '
                f'mtype={self.mtype!s}, '
                f'uri={self.uri!s}, '
                f'total_episodes={self.total_episodes!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.episode_object import EpisodeObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_episodes import ManyEpisodes
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_saved_episode_object import PagingSavedEpisodeObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class EpisodesController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(EpisodesController, self).__init__(config)

    def get_an_episode(self,
                       id,
                       market=None):
        """Does a GET request to /episodes/{id}.

        Get Spotify catalog information for a single episode identified by its
        unique Spotify ID.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.

        Returns:
            EpisodeObject: Response from the API. An episode

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/episodes/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(EpisodeObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_multiple_episodes(self,
                              ids,
                              market=None):
        """Does a GET request to /episodes.

        Get Spotify catalog information for several episodes based on their
        Spotify IDs.

        Args:
            ids (str): The request query parameter.
            market (str, optional): The request query parameter.

        Returns:
            ManyEpisodes: Response from the API. A set of episodes

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/episodes')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyEpisodes.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_saved_episodes(self,
                                 market=None,
                                 limit=20,
                                 offset=0):
        """Does a GET request to /me/episodes.

        Get a list of the episodes saved in the current Spotify user's
        library.<br/>
        This API endpoint is in __beta__ and could change without warning.
        Please share any feedback that you have, or issues that you discover,
        in our [developer community
        forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spo
        tify_Developer).

        Args:
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingSavedEpisodeObject: Response from the API. Pages of episodes

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/episodes')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSavedEpisodeObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def save_episodes_user(self,
                           ids,
                           body=None):
        """Does a PUT request to /me/episodes.

        Save one or more episodes to the current user's library.<br/>
        This API endpoint is in __beta__ and could change without warning.
        Please share any feedback that you have, or issues that you discover,
        in our [developer community
        forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spo
        tify_Developer).

        Args:
            ids (str): The request query parameter.
            body (MeEpisodesRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Episode saved

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/episodes')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def remove_episodes_user(self,
                             ids,
                             body=None):
        """Does a DELETE request to /me/episodes.

        Remove one or more episodes from the current user's library.<br/>
        This API endpoint is in __beta__ and could change without warning.
        Please share any feedback that you have, or issues that you discover,
        in our [developer community
        forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spo
        tify_Developer).

        Args:
            ids (str): The request query parameter.
            body (MeEpisodesRequest1, optional): The request body parameter.

        Returns:
            void: Response from the API. Episode removed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/episodes')
            .http_method(HttpMethodEnum.DELETE)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def check_users_saved_episodes(self,
                                   ids):
        """Does a GET request to /me/episodes/contains.

        Check if one or more episodes is already saved in the current Spotify
        user's 'Your Episodes' library.<br/>
        This API endpoint is in __beta__ and could change without warning.
        Please share any feedback that you have, or issues that you discover,
        in our [developer community
        forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spo
        tify_Developer)..

        Args:
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/episodes/contains')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.saved_audiobook_object import SavedAudiobookObject


class PagingSavedAudiobookObject(object):

    """Implementation of the 'PagingSavedAudiobookObject' model.

    Attributes:
        href (str): A link to the Web API endpoint returning the full result
            of the request
        limit (int): The maximum number of items in the response (as set in
            the query or by default).
        next (str): URL to the next page of items. ( `null` if none)
        offset (int): The offset of the items returned (as set in the query or
            by default)
        previous (str): URL to the previous page of items. ( `null` if none)
        total (int): The total number of items available to return.
        items (List[SavedAudiobookObject]): The model property of type
            List[SavedAudiobookObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "href": 'href',
        "limit": 'limit',
        "next": 'next',
        "offset": 'offset',
        "previous": 'previous',
        "total": 'total',
        "items": 'items'
    }

    _nullables = [
        'next',
        'previous',
    ]

    def __init__(self,
                 href=None,
                 limit=None,
                 next=None,
                 offset=None,
                 previous=None,
                 total=None,
                 items=None):
        """Constructor for the PagingSavedAudiobookObject class"""

        # Initialize members of the class
        self.href = href 
        self.limit = limit 
        self.next = next 
        self.offset = offset 
        self.previous = previous 
        self.total = total 
        self.items = items 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        href = dictionary.get("href") if dictionary.get("href") else None
        limit = dictionary.get("limit") if dictionary.get("limit") else None
        next = dictionary.get("next") if dictionary.get("next") else None
        offset = dictionary.get("offset") if dictionary.get("offset") else None
        previous = dictionary.get("previous") if dictionary.get("previous") else None
        total = dictionary.get("total") if dictionary.get("total") else None
        items = None
        if dictionary.get('items') is not None:
            items = [SavedAudiobookObject.from_dictionary(x) for x in dictionary.get('items')]
        # Return an object of this model
        return cls(href,
                   limit,
                   next,
                   offset,
                   previous,
                   total,
                   items)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'href={self.href!r}, '
                f'limit={self.limit!r}, '
                f'next={self.next!r}, '
                f'offset={self.offset!r}, '
                f'previous={self.previous!r}, '
                f'total={self.total!r}, '
                f'items={self.items!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'href={self.href!s}, '
                f'limit={self.limit!s}, '
                f'next={self.next!s}, '
                f'offset={self.offset!s}, '
                f'previous={self.previous!s}, '
                f'total={self.total!s}, '
                f'items={self.items!s})')
